@@ -1,7 +1,6 @@
 /*
 
-    $Id$
-    Copyright (C) 2007,2008,2009,2010 Alois Schloegl <a.schloegl@ieee.org>
+    Copyright (C) 2007,2008,2009,2010,2013,2015 Alois Schloegl <alois.schloegl@ist.ac.at>
     This file is part of the "BioSig for C/C++" repository 
     (biosig4c++) at http://biosig.sf.net/ 
 
@@ -16,16 +15,33 @@
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
+#ifdef HAVE_CHOLMOD
+#  if !defined(__APPLE__)
+#    include <suitesparse/cholmod.h>
+#  else
+#    include <cholmod.h>
+#  endif
+#endif
+#include <biosig-dev.h>
+#include <biosig.h>
+
+#ifdef NDEBUG
+#define VERBOSE_LEVEL 0 	// turn off debugging information, but its only used without NDEBUG
+#else
+extern int VERBOSE_LEVEL; 	// used for debugging
+#endif
+
 
-#include "../biosig-dev.h"
 #ifdef tmwtypes_h
   #if (MX_API_VER<=0x07020000)
     typedef int mwSize;
   #endif 
 #endif 
 
-typedef mwSize Int;
+#ifndef TRUE
 #define TRUE (1)
+#endif
 
 #ifdef CHOLMOD_H
 //#include "cholmod/matlab/cholmod_matlab.h"
@@ -76,15 +92,15 @@ cholmod_sparse *sputil_get_sparse
     const mxArray *Amatlab, /* MATLAB version of the matrix */
     cholmod_sparse *A,	    /* CHOLMOD version of the matrix */
     double *dummy,	    /* a pointer to a valid scalar double */
-    Int stype		    /* -1: lower, 0: unsymmetric, 1: upper */
+    mwSize stype		    /* -1: lower, 0: unsymmetric, 1: upper */
 )
 {
-    Int *Ap ;
+    mwSize *Ap ;
     A->nrow = mxGetM (Amatlab) ;
     A->ncol = mxGetN (Amatlab) ;
-    A->p = (Int *) mxGetJc (Amatlab) ;
-    A->i = (Int *) mxGetIr (Amatlab) ;
-    Ap = (Int*)A->p ;
+    A->p = (mwSize *) mxGetJc (Amatlab) ;
+    A->i = (mwSize *) mxGetIr (Amatlab) ;
+    Ap = (mwSize*)A->p ;
     A->nzmax = Ap [A->ncol] ;
     A->packed = TRUE ;
     A->sorted = TRUE ;
@@ -199,7 +215,9 @@ void mexFunction(
 		mexPrintf("   Usage of mexSLOAD:\n");
 		mexPrintf("\t[s,HDR]=mexSLOAD(f)\n");
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan)\n\t\tchan must be sorted in ascending order\n");
+#ifdef CHOLMOD_H
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,ReRef)\n\t\treref is a (sparse) matrix for rerefencing\n");
+#endif
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan,'...')\n");
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan,'OVERFLOWDETECTION:ON')\n");
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan,'OVERFLOWDETECTION:OFF')\n");
@@ -245,11 +263,13 @@ void mexFunction(
 			mexPrintf("arg[%i] IsStruct\n",k);
 #endif
 		}
-#ifdef CHOLMOD_H
 		else if ((k==1) && mxIsSparse(arg)) {
+#ifdef CHOLMOD_H
 			rr = sputil_get_sparse(arg,&RR,&dummy,0);
-		}
+#else
+			mexErrMsgTxt("This version of mexSLOAD does not support re-referencing matrix - recompile with -DWITH_CHOLMOD -lcholmod \n");
 #endif 
+		}
 		else if ((k==1) && mxIsNumeric(arg)) {
 #ifdef DEBUG		
 			mexPrintf("arg[%i] IsNumeric\n",k);
@@ -277,13 +297,13 @@ void mexFunction(
 //				FlagMXclass = mxSINGLE_CLASS;
 			else if (!strncmp(mxArrayToString(prhs[k]),"TARGETSEGMENT:",14))
 				TARGETSEGMENT = atoi(mxArrayToString(prhs[k])+14);
-			else if (!strcmpi(mxArrayToString(prhs[k]), "SWEEP") && (prhs[k+1] != NULL) && mxIsNumeric(prhs[k+1]))
+			else if (!strcasecmp(mxArrayToString(prhs[k]), "SWEEP") && (prhs[k+1] != NULL) && mxIsNumeric(prhs[k+1]))
 				argSweepSel = ++k;
 		}
 		else {
 #ifndef mexSOPEN
 			mexPrintf("mexSLOAD: argument #%i is invalid.",k+1);	
-			mexErrMsgTxt("mexSLOAD failes because of unknown parameter\n");	
+			mexErrMsgTxt("mexSLOAD fails because of unknown parameter\n");
 #else
 			mexPrintf("mexSOPEN: argument #%i is invalid.",k+1);	
 			mexErrMsgTxt("mexSOPEN fails because of unknown parameter\n");	
@@ -295,6 +315,32 @@ void mexFunction(
 		mexPrintf("110: input arguments checked\n");
 
 	hdr = constructHDR(0,0);
+
+#ifdef __LIBBIOSIG2_H__
+
+	unsigned flags = (!!FlagOverflowDetection)*BIOSIG_FLAG_OVERFLOWDETECTION + (!!FlagUCAL)*BIOSIG_FLAG_UCAL;
+#ifdef CHOLMOD_H
+	flags += (rr!=NULL)*BIOSIG_FLAG_ROW_BASED_CHANNELS;
+#else
+	biosig_reset_flag(hdr, BIOSIG_FLAG_ROW_BASED_CHANNELS);
+#endif
+	biosig_set_flag(hdr, flags);
+
+	biosig_set_targetsegment(hdr, TARGETSEGMENT);
+
+	// sweep selection for Heka format
+	if (argSweepSel>0) {
+		double *SZ     = (double*) mxGetData(prhs[argSweepSel]);
+		k = 0;
+		while (k < mxGetNumberOfElements(prhs[argSweepSel]) && k < 5) {
+			biosig_set_segment_selection(hdr, k+1, (uint32_t)SZ[k]);
+			k++;
+		}
+	}
+
+#else //__LIBBIOSIG2_H__
+
+
 	hdr->FLAG.OVERFLOWDETECTION = FlagOverflowDetection; 
 	hdr->FLAG.UCAL = FlagUCAL;
 #ifdef CHOLMOD_H
@@ -314,6 +360,10 @@ void mexFunction(
 		}
 	}
 
+
+#endif // __LIBBIOSIG2_H__ : TODO: below, nothing is converted to level-2 interface, yet
+
+
 	if (VERBOSE_LEVEL>7) 
 		mexPrintf("120: going to sopen\n");
 
@@ -333,7 +383,15 @@ void mexFunction(
 
 		const char* fields[]={"TYPE","VERSION","FileName","FLAG","ErrNum","ErrMsg"};
 		HDR = mxCreateStructMatrix(1, 1, 6, fields);
+#ifdef __LIBBIOSIG2_H__
+		mxSetField(HDR,0,"FileName",mxCreateString(biosig_get_filename(hdr)));
+		const char *FileTypeString = GetFileTypeString(biosig_get_filetype(hdr));
+		mxSetField(HDR,0,"VERSION",mxCreateDoubleScalar(biosig_get_version(hdr)));
+#else
 		mxSetField(HDR,0,"FileName",mxCreateString(hdr->FileName));
+		const char *FileTypeString = GetFileTypeString(hdr->TYPE);
+		mxSetField(HDR,0,"VERSION",mxCreateDoubleScalar(hdr->VERSION));
+#endif
 		mxArray *errnum = mxCreateNumericMatrix(1,1,mxUINT8_CLASS,mxREAL);
 		*(uint8_t*)mxGetData(errnum) = (uint8_t)status;
 		mxSetField(HDR,0,"ErrNum",errnum);
@@ -341,15 +399,10 @@ void mexFunction(
 #ifdef HAVE_OCTAVE
 		// handle bug in octave: mxCreateString(NULL) causes segmentation fault
 		// Octave 3.2.3 causes a seg-fault in mxCreateString(NULL)
-		 
-		{
-		const char *p = GetFileTypeString(hdr->TYPE);
-		if (p) mxSetField(HDR,0,"TYPE",mxCreateString(p));
-		}
-#else
-		mxSetField(HDR,0,"TYPE",mxCreateString(GetFileTypeString(hdr->TYPE)));
+		if (FileTypeString) FileTypeString="\0";
 #endif
-		mxSetField(HDR,0,"VERSION",mxCreateDoubleScalar(hdr->VERSION));
+		mxSetField(HDR,0,"TYPE",mxCreateString(FileTypeString));
+
 
 		char *msg = (char*)malloc(72+23+strlen(FileName)); // 72: max length of constant text, 23: max length of GetFileTypeString()
 		if (msg == NULL) 
@@ -358,11 +411,11 @@ void mexFunction(
 		    if (status==B4C_CANNOT_OPEN_FILE)
 			sprintf(msg,"Error mexSLOAD: file %s not found.\n",FileName);		/* Flawfinder: ignore *** sufficient memory is allocated above */
 		    else if (status==B4C_FORMAT_UNKNOWN)
-			sprintf(msg,"Error mexSLOAD: Cannot open file %s - format %s not known.\n",FileName,GetFileTypeString(hdr->TYPE));	/* Flawfinder: ignore *** sufficient memory is allocated above */
+			sprintf(msg,"Error mexSLOAD: Cannot open file %s - format %s not known.\n",FileName,FileTypeString);	/* Flawfinder: ignore *** sufficient memory is allocated above */
 		    else if (status==B4C_FORMAT_UNSUPPORTED)
-			sprintf(msg,"Error mexSLOAD: Cannot open file %s - format %s not supported [%s].\n", FileName, GetFileTypeString(hdr->TYPE), hdr->AS.B4C_ERRMSG);	/* Flawfinder: ignore *** sufficient memory is allocated above */
+			sprintf(msg,"Error mexSLOAD: Cannot open file %s - format %s not supported [%s].\n", FileName, FileTypeString, hdr->AS.B4C_ERRMSG);	/* Flawfinder: ignore *** sufficient memory is allocated above */
 		    else 	
-			sprintf(msg,"Error %i mexSLOAD: Cannot open file %s - format %s not supported [%s].\n", status, FileName, GetFileTypeString(hdr->TYPE), hdr->AS.B4C_ERRMSG); 	/* Flawfinder: ignore *** sufficient memory is allocated above */
+			sprintf(msg,"Error %i mexSLOAD: Cannot open file %s - format %s not supported [%s].\n", status, FileName, FileTypeString, hdr->AS.B4C_ERRMSG); 	/* Flawfinder: ignore *** sufficient memory is allocated above */
 			
 		    mxSetField(HDR,0,"ErrMsg",mxCreateString(msg));
 		    free(msg);
@@ -451,7 +504,7 @@ void mexFunction(
 	if ((status=serror2(hdr))) return;  
 
 	if (VERBOSE_LEVEL>7) 
-		fprintf(stderr,"\n[129] SREAD/SCLOSE on %s successful [%i,%i] [%Li,%i] %i.\n",hdr->FileName,hdr->data.size[0],hdr->data.size[1],hdr->NRec,count,NS);
+		fprintf(stderr,"\n[129] SREAD/SCLOSE on %s successful [%i,%i] [%i,%i] %i.\n",hdr->FileName,(int)hdr->data.size[0],(int)hdr->data.size[1],(int)hdr->NRec,(int)count,(int)NS);
 
 
 //	hdr2ascii(hdr,stderr,4);	
@@ -465,11 +518,11 @@ void mexFunction(
 
 		mxArray *tmp, *tmp2, *Patient, *Manufacturer, *ID, *EVENT, *Filter, *Flag, *FileType;
 		uint16_t numfields;
-		const char *fnames[] = {"TYPE","VERSION","FileName","T0","FILE","Patient",\
+		const char *fnames[] = {"TYPE","VERSION","FileName","T0","tzmin","Patient",\
 		"HeadLen","NS","SPR","NRec","SampleRate", "FLAG", \
 		"EVENT","Label","LeadIdCode","PhysDimCode","PhysDim","Filter",\
 		"PhysMax","PhysMin","DigMax","DigMin","Transducer","Cal","Off","GDFTYP","TOffset",\
-		"LowPass","HighPass","Notch","ELEC","Impedance","fZ","AS","Dur","REC","Manufacturer",NULL};
+		"ELEC","Impedance","fZ","AS","Dur","REC","Manufacturer",NULL};
 
 		for (numfields=0; fnames[numfields++] != NULL; );
 		HDR = mxCreateStructMatrix(1, 1, --numfields, fnames);
@@ -485,6 +538,7 @@ void mexFunction(
 		mxSetField(HDR,0,"FileName",mxCreateString(hdr->FileName));
                 
 		mxSetField(HDR,0,"T0",mxCreateDoubleScalar(ldexp(hdr->T0,-32)));
+		mxSetField(HDR,0,"tzmin",mxCreateDoubleScalar(hdr->tzmin));
 
 		/* Channel information */ 
 #ifdef CHOLMOD_H
@@ -547,8 +601,8 @@ void mexFunction(
 			*(mxGetPr(ELEC_Orient)+k1+NS*2)	= (double)hdr->CHANNEL[k].Orientation[2];
 			*(mxGetPr(ELEC_Area)+k1)	= (double)hdr->CHANNEL[k].Area;
 */
-			mxSetCell(Label,k1,mxCreateString(hdr->CHANNEL[k].Label));
-			mxSetCell(Transducer,k1,mxCreateString(hdr->CHANNEL[k].Transducer));
+			mxSetCell(Label,k1,mxCreateString(hdr->CHANNEL[k].Label ? hdr->CHANNEL[k].Label : ""));
+			mxSetCell(Transducer,k1,mxCreateString(hdr->CHANNEL[k].Transducer ? hdr->CHANNEL[k].Transducer : ""));
 			
 			mxSetCell(PhysDim1,k1,mxCreateString(PhysDim3(hdr->CHANNEL[k].PhysDimCode)));
 			k1++;
@@ -571,7 +625,7 @@ void mexFunction(
 		mxSetField(HDR,0,"Transducer",Transducer);
 		mxSetField(HDR,0,"Label",Label);
 
-		const char* field[] = {"XYZ","Orientation","Area","GND","REF",NULL};
+		const char* field[] = {"XYZ",NULL};
 		for (numfields=0; field[numfields++] != 0; );
 		tmp = mxCreateStructMatrix(1, 1, --numfields, field);
 		mxSetField(tmp,0,"XYZ",ELEC_POS);
@@ -650,12 +704,24 @@ void mexFunction(
 
 		mxArray *TYP = mxCreateDoubleMatrix(hdr->EVENT.N,1, mxREAL);
 		mxArray *POS = mxCreateDoubleMatrix(hdr->EVENT.N,1, mxREAL);
+
 		for (k=0; k<hdr->EVENT.N; ++k) {
 			*(mxGetPr(TYP)+k) = (double)hdr->EVENT.TYP[k];
 			*(mxGetPr(POS)+k) = (double)hdr->EVENT.POS[k]+1;   // conversion from 0-based to 1-based indexing 
 		} 
 		mxSetField(EVENT,0,"TYP",TYP);
 		mxSetField(EVENT,0,"POS",POS);
+
+#if (BIOSIG_VERSION >= 10500)
+		if (hdr->EVENT.TimeStamp) {
+			mxArray *TimeStamp = mxCreateDoubleMatrix(hdr->EVENT.N,1, mxREAL);
+			for (k=0; k<hdr->EVENT.N; ++k) {
+				*(mxGetPr(TimeStamp)+k) = ldexp(hdr->EVENT.TimeStamp[k],-32);
+			} 
+			mxAddField(EVENT, "TimeStamp");
+			mxSetField(EVENT,0,"TimeStamp",TimeStamp);
+		}	
+#endif
 		mxSetField(EVENT,0,"SampleRate",mxCreateDoubleScalar(hdr->EVENT.SampleRate));
 		mxSetField(HDR,0,"EVENT",EVENT);
 
@@ -680,15 +746,20 @@ void mexFunction(
 		const char *patient_fields[] = {"Sex","Handedness","Id","Name","Weight","Height","Birthday",NULL};
 		for (numfields=0; patient_fields[numfields++] != 0; );
 		Patient = mxCreateStructMatrix(1, 1, --numfields, patient_fields);
-		const char *strarray[1];
-		if (hdr->Patient.Name) {
-			strarray[0] = hdr->Patient.Name; 
-			mxSetField(Patient,0,"Name",mxCreateCharMatrixFromStrings(1,strarray));
-		}	
-		if (hdr->Patient.Id) {
-			strarray[0] = hdr->Patient.Id; 
-			mxSetField(Patient,0,"Id",mxCreateCharMatrixFromStrings(1,strarray));
-		}	
+		const char *strarray;
+#ifdef __LIBBIOSIG2_H__
+		strarray = biosig_get_patient_name(hdr);
+		mxSetField(Patient,0,"Name",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+
+		strarray = biosig_get_patient_id(hdr);
+		mxSetField(Patient,0,"Id",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+#else
+		strarray = hdr->Patient.Name;
+		mxSetField(Patient,0,"Name",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+
+		strarray = hdr->Patient.Id;
+		mxSetField(Patient,0,"Id",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+#endif
 		mxSetField(Patient,0,"Handedness",mxCreateDoubleScalar(hdr->Patient.Handedness));
 
 		mxSetField(Patient,0,"Sex",mxCreateDoubleScalar(hdr->Patient.Sex));
@@ -711,24 +782,36 @@ void mexFunction(
 		const char *manufacturer_fields[] = {"Name","Model","Version","SerialNumber",NULL};
 		for (numfields=0; manufacturer_fields[numfields++] != 0; );
 		Manufacturer = mxCreateStructMatrix(1, 1, --numfields, manufacturer_fields);
-		if (hdr->ID.Manufacturer.Name) {
-			strarray[0] = hdr->ID.Manufacturer.Name;
-			mxSetField(Manufacturer,0,"Name",mxCreateCharMatrixFromStrings(1,strarray));
-		}	
-		if (hdr->ID.Manufacturer.Model) {
-			strarray[0] = hdr->ID.Manufacturer.Model;
-			mxSetField(Manufacturer,0,"Model",mxCreateCharMatrixFromStrings(1,strarray));
-		}	
-		if (hdr->ID.Manufacturer.Version) {
-			strarray[0] = hdr->ID.Manufacturer.Version;
-			mxSetField(Manufacturer,0,"Version",mxCreateCharMatrixFromStrings(1,strarray));
-		}	
-		if (hdr->ID.Manufacturer.SerialNumber) {
-			strarray[0] = hdr->ID.Manufacturer.SerialNumber;
-			mxSetField(Manufacturer,0,"SerialNumber",mxCreateCharMatrixFromStrings(1,strarray));
-		}
+
+#ifdef __LIBBIOSIG2_H__
+		strarray = biosig_get_manufacturer_name(hdr);
+		mxSetField(Manufacturer,0,"Name",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+
+		strarray = biosig_get_manufacturer_model(hdr);
+		mxSetField(Manufacturer,0,"Model",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+
+		strarray = biosig_get_manufacturer_version(hdr);
+		mxSetField(Manufacturer,0,"Version",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+
+		strarray = biosig_get_manufacturer_serial_number(hdr);
+		mxSetField(Manufacturer,0,"SerialNumber",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+
+#else
+		strarray = hdr->ID.Manufacturer.Name;
+		mxSetField(Manufacturer,0,"Name",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+
+		strarray = hdr->ID.Manufacturer.Model;
+		mxSetField(Manufacturer,0,"Model",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+
+		strarray = hdr->ID.Manufacturer.Version;
+		mxSetField(Manufacturer,0,"Version",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+
+		strarray = hdr->ID.Manufacturer.SerialNumber;
+		mxSetField(Manufacturer,0,"SerialNumber",mxCreateCharMatrixFromStrings(strarray!=NULL, &strarray));
+#endif
 		mxSetField(HDR,0,"Manufacturer",Manufacturer);
 
+
 	if (VERBOSE_LEVEL>7) 
 		fprintf(stdout,"[148] going for SCLOSE\n");
 
