@@ -1,7 +1,6 @@
 /*
 
-    $Id$
-    Copyright (C) 2000,2005,2007-2012 Alois Schloegl <alois.schloegl@gmail.com>
+    Copyright (C) 2000,2005,2007-2016 Alois Schloegl <alois.schloegl@gmail.com>
     Copyright (C) 2007 Elias Apostolopoulos
     This file is part of the "BioSig for C/C++" repository 
     (biosig4c++) at http://biosig.sf.net/ 
@@ -26,12 +25,18 @@
 #define T1T2
 
 
+#include <assert.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <time.h>
 #include "biosig-dev.h"
+#include "biosig.h"
+
+#define min(a,b)        (((a) < (b)) ? (a) : (b))
+#define max(a,b)        (((a) > (b)) ? (a) : (b))
 
 #ifdef __cplusplus
 extern "C" {
@@ -49,16 +54,24 @@ int main(int argc, char **argv){
     
     HDRTYPE 	*hdr; 
     size_t 	count, k1, ne=0;
-    uint16_t 	numopt = 0;
     char 	*source, *dest, *tmpstr; 
-    enum FileFormat SOURCE_TYPE, TARGET_TYPE=GDF; 		// type of file format
+    enum FileFormat SOURCE_TYPE; 		// type of file format
+    struct {
+	enum FileFormat TYPE;
+	float VERSION;
+    } TARGET;
+    TARGET.TYPE=GDF;
+    TARGET.VERSION=-1.0;
+
     int		COMPRESSION_LEVEL=0;
     int		status; 	
     uint16_t	k;
     int		TARGETSEGMENT=1; 	// select segment in multi-segment file format EEG1100 (Nihon Kohden)
-    int 	VERBOSE	= 1; 
-    char	FLAG_CNT32 = 0; 	// assume CNT format is 16bit
+    int 	VERBOSE	= 0; 
+    char	FLAG_ANON = 1;
+    char	FLAG_CSV = 0;
     char	FLAG_JSON = 0; 
+    char	FLAG_DYGRAPH = 0; 
     char	*argsweep = NULL;
     double	t1=0.0, t2=1.0/0.0;
 #ifdef CHOLMOD_H
@@ -68,8 +81,8 @@ int main(int argc, char **argv){
 	
     for (k=1; k<argc; k++) {
     	if (!strcmp(argv[k],"-v") || !strcmp(argv[k],"--version") ) {
-		fprintf(stdout,"save2gdf (BioSig4C++) v%04.2f\n", BIOSIG_VERSION*1e-4);
-		fprintf(stdout,"Copyright (C) 2006,2007,2008,2009,2010,2011 by Alois Schloegl and others\n");
+		fprintf(stdout,"save2gdf (BioSig4C++) v%i.%i.%i\n", BIOSIG_VERSION_MAJOR, BIOSIG_VERSION_MINOR, BIOSIG_PATCHLEVEL);
+		fprintf(stdout,"Copyright (C) 2006-2013 by Alois Schloegl and others\n");
 		fprintf(stdout,"This file is part of BioSig http://biosig.sf.net - the free and\n");
 		fprintf(stdout,"open source software library for biomedical signal processing.\n\n");
 		fprintf(stdout,"BioSig is free software; you can redistribute it and/or modify\n");
@@ -88,17 +101,31 @@ int main(int argc, char **argv){
 		fprintf(stdout,"   -v, --version\n\tprints version information\n");
 		fprintf(stdout,"   -h, --help   \n\tprints this information\n");
 #ifdef T1T2
-		fprintf(stdout,"   [t0,dt]\n\tstart time and duriation in seconds (do not use any spaces). \n");
+		fprintf(stdout,"   [t0,dt]\n\tstart time and duration in seconds (do not use any spaces). \n");
 		fprintf(stdout,"\tTHIS FEATURE IS CURRENTLY EXPERIMENTAL !!!\n");
 #endif
 #ifdef CHOLMOD_H
 		fprintf(stdout,"   -r, --ref=MM  \n\trereference data with matrix file MM. \n\tMM must be a 'MatrixMarket matrix coordinate real general' file.\n");
 #endif
-		fprintf(stdout,"   -cnt32\n\tmust be set for reading 32 bit CNT files\n");
 		fprintf(stdout,"   -f=FMT  \n\tconverts data into format FMT\n");
-		fprintf(stdout,"   -JSON  \n\tshows header, and events in JSON format\n");
-		fprintf(stdout,"\tFMT must represent a valid target file format\n"); 
-		fprintf(stdout,"\tCurrently are supported: HL7aECG, SCP_ECG (EN1064), GDF, EDF, BDF, CFWB, BIN, ASCII, BVA (BrainVision)\n\tas well as HEKA v2 -> ITX\n"); 
+		fprintf(stdout,"\tFMT must represent a valid target file format\n");
+#if defined(WITH_SCP3)
+		fprintf(stdout,"\tCurrently are supported: HL7aECG, SCP_ECG (EN1064:2005), SCP2, SCP3, GDF, EDF, BDF, CFWB, BIN, ASCII, ATF, BVA (BrainVision)\n\tas well as HEKA v2 -> ITX\n");
+#else
+		fprintf(stdout,"\tCurrently are supported: HL7aECG, SCP_ECG (EN1064:2005), GDF, EDF, BDF, CFWB, BIN, ASCII, ATF, BVA (BrainVision)\n\tas well as HEKA v2 -> ITX\n");
+#endif
+		fprintf(stdout, "   -a, --anon=yes   (default)\n"
+				"\tanonymized data processing - personalize data (name, and birthday) is not processed but ignored.\n"
+				"\tThe patient can be still identified with the unique patient identifier (and an external database).\n"
+				"\tThis is for many cases sufficient (e.g. for research etc.). This mode can be turn off with\n"
+				"   -n, --anon=no\n"
+				"\tThis will process personal information like name and birthday. One might want to use this mode\n"
+				"\twhen converting personalized patient data and no unique patient identifier is available.\n"
+				"\tIt's recommended to pseudonize the data, or to use the patient identifier instead of patient name and birthday.\n"
+		);
+		fprintf(stdout,"   -CSV  \n\texports data into CSV file\n");
+		fprintf(stdout,"   -DYGRAPH, -f=DYGRAPH  \n\tproduces JSON output for presentation with dygraphs\n");
+		fprintf(stdout,"   -JSON  \n\tshows header and events in JSON format\n");
 		fprintf(stdout,"   -z=#, -z#\n\t# indicates the compression level (#=0 no compression; #=9 best compression, default #=1)\n");
 		fprintf(stdout,"   -s=#\tselect target segment # (in the multisegment file format EEG1100)\n");
 		fprintf(stdout,"   -SWEEP=ne,ng,ns\n\tsweep selection of HEKA/PM files\n\tne,ng, and ns select the number of experiment, the number of group, and the sweep number, resp.\n");
@@ -120,46 +147,91 @@ int main(int argc, char **argv){
 	}
     	else if (!strncmp(argv[k],"-VERBOSE",2))  {
 	    	VERBOSE = argv[k][strlen(argv[k])-1]-48;
-#ifndef VERBOSE_LEVEL
+#ifndef NDEBUG
 	// then VERBOSE_LEVEL is not a constant but a variable
 	VERBOSE_LEVEL = VERBOSE; 
 #endif
 	}
-    	else if (!strncmpi(argv[k],"-SWEEP=",7))  {
+    	else if (!strncasecmp(argv[k],"-SWEEP=",7))  {
 	    	argsweep = argv[k]+6;
 
 	}
-	else if (!strcmpi(argv[k],"-JSON"))
+
+	else if (!strcasecmp(argv[k],"-a") || !strcasecmp(argv[k],"--anon") )
+		FLAG_ANON = 1;
+
+	else if (!strcasecmp(argv[k],"-n") || !strcasecmp(argv[k],"--anon=no") )
+		FLAG_ANON = 0;
+
+	else if (!strcasecmp(argv[k],"-CSV"))
+		FLAG_CSV = 1;
+
+	else if (!strcasecmp(argv[k],"-JSON"))
 		FLAG_JSON = 1;
 
+	else if (!strcasecmp(argv[k],"-DYGRAPH"))
+		FLAG_DYGRAPH = 1;
+
     	else if (!strncmp(argv[k],"-f=",3))  	{
     		if (0) {}
     		else if (!strncmp(argv[k],"-f=ASCII",8))
-			TARGET_TYPE=ASCII;
+			TARGET.TYPE=ASCII;
+		else if (!strcmp(argv[k],"-f=ATF"))
+			TARGET.TYPE=ATF;
     		else if (!strcmp(argv[k],"-f=BDF"))
-			TARGET_TYPE=BDF;
+			TARGET.TYPE=BDF;
     		else if (!strncmp(argv[k],"-f=BIN",6))
-			TARGET_TYPE=BIN;
+			TARGET.TYPE=BIN;
     		else if (!strncmp(argv[k],"-f=BVA",6))
-			TARGET_TYPE=BrainVision;
+			TARGET.TYPE=BrainVision;
     		else if (!strncmp(argv[k],"-f=CFWB",7))
-			TARGET_TYPE=CFWB;
+			TARGET.TYPE=CFWB;
     		else if (!strcmp(argv[k],"-f=EDF"))
-			TARGET_TYPE=EDF;
+			TARGET.TYPE=EDF;
     	 	else if (!strcmp(argv[k],"-f=GDF"))
-			TARGET_TYPE=GDF;
-    		else if (!strcmp(argv[k],"-f=GDF1"))
-			TARGET_TYPE=GDF1;
+			TARGET.TYPE=GDF;
+		else if (!strcmp(argv[k],"-f=GDF1")) {
+			TARGET.TYPE=GDF;
+			TARGET.VERSION=1.0;
+		}
+		else if (!strcmp(argv[k],"-f=GDF2")) {
+			TARGET.TYPE=GDF;
+			TARGET.VERSION=2.0;
+		}
+#if (BIOSIG_VERSION >= 10700)
+		else if (!strcmp(argv[k],"-f=GDF3") && (get_biosig_version() > 0x010703) ) {
+			TARGET.TYPE=GDF;
+			TARGET.VERSION=3.0;
+		}
+#endif
     		else if (!strncmp(argv[k],"-f=HL7",6))
-			TARGET_TYPE=HL7aECG;
+			TARGET.TYPE=HL7aECG;
+		else if (!strncmp(argv[k],"-f=CSV",6)) {
+			FLAG_CSV = 1;
+		}
+		else if (!strncmp(argv[k],"-f=DYGRAPH",10)) {
+			FLAG_DYGRAPH = 1;
+		}
     		else if (!strncmp(argv[k],"-f=MFER",7))
-			TARGET_TYPE=MFER;
-    		else if (!strncmp(argv[k],"-f=SCP",6))
-			TARGET_TYPE=SCP_ECG;
+			TARGET.TYPE=MFER;
+#if defined(WITH_SCP3)
+		else if (!strncmp(argv[k],"-f=SCP3",7) && (get_biosig_version() > 0x010806) ) {
+			TARGET.TYPE=SCP_ECG;
+			TARGET.VERSION=3.0;
+			fprintf(stderr,"WARNING %s: Specification of SCPv3 is not finalized, "
+				"\n\tand is subject to change without further notice. "
+				"\n\tSCP3 may be used only for experimental work"
+				"\n\tYou are warned !!!\n",__FILE__);
+		}
+#endif
+		else if (!strncmp(argv[k],"-f=SCP",6)) {
+			TARGET.TYPE=SCP_ECG;
+			TARGET.VERSION=2.0;
+		}
 //    		else if (!strncmp(argv[k],"-f=TMSi",7))
-//			TARGET_TYPE=TMSiLOG;
+//			TARGET.TYPE=TMSiLOG;
     		else if (!strncmp(argv[k],"-f=ITX",6))
-			TARGET_TYPE=ITX;
+			TARGET.TYPE=ITX;
 		else {
 			fprintf(stderr,"format %s not supported.\n",argv[k]);
 			return(-1);
@@ -177,10 +249,6 @@ int main(int argc, char **argv){
     		TARGETSEGMENT = atoi(argv[k]+3);
 	}
 
-    	else if (!strncmp(argv[k],"-cnt32",3))  {
-    		FLAG_CNT32 = 1;
-	}
-
     	else if (argv[k][0]=='[' && argv[k][strlen(argv[k])-1]==']' && (tmpstr=strchr(argv[k],',')) )  	{
 		t1 = strtod(argv[k]+1,NULL);
 		t2 = strtod(tmpstr+1,NULL);
@@ -188,41 +256,40 @@ int main(int argc, char **argv){
 	}
 	
 	else {
-		numopt = k-1;
 		break; 		
 	}
 
 	if (VERBOSE_LEVEL>7) 
-		fprintf(stdout,"[103] save2gdf: arg%i = <%s>\n", k, argv[k]);
+		fprintf(stdout,"%s (line %i): save2gdf: arg%i = <%s>\n",__FILE__,__LINE__, k, argv[k]);
 
     }
 
 
 	source = NULL;
 	dest = NULL;
-    	switch (argc - numopt) {
-    	case 1:
+
+	switch (argc - k) {
+	case 0:
 		fprintf(stderr,"save2gdf: missing file argument\n");
 		fprintf(stdout,"usage: save2gdf [options] SOURCE DEST\n");
 		fprintf(stdout," for more details see also save2gdf --help \n");
 		exit(-1);
-    	case 3:
-    		dest   = argv[numopt+2]; 
     	case 2:
-	    	source = argv[numopt+1]; 
+		dest   = argv[k+1];
+	case 1:
+		source = argv[k];
     	}	
 
 	if (VERBOSE_LEVEL<0) VERBOSE=1; // default 
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"[108] SAVE2GDF %s %s started \n", source, dest
-);
+	if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): SAVE2GDF %s %s started \n",__FILE__,__LINE__, source, dest);
+	fprintf(stderr,"%s %s %s\n", argv[0], source, dest);
 
 	tzset();
 	hdr = constructHDR(0,0);
 	// hdr->FLAG.OVERFLOWDETECTION = FlagOverflowDetection; 
-	hdr->FLAG.UCAL = ((TARGET_TYPE==BIN) || (TARGET_TYPE==ASCII));
+	hdr->FLAG.UCAL = ((TARGET.TYPE==BIN) || (TARGET.TYPE==SCP_ECG));
 	hdr->FLAG.TARGETSEGMENT = TARGETSEGMENT;
-	hdr->FLAG.CNT32 = FLAG_CNT32;
-	// hdr->FLAG.ANONYMOUS = 0; 	// personal names are processed 
+	hdr->FLAG.ANONYMOUS = FLAG_ANON;
 	
 	if (argsweep) {
 		k = 0;
@@ -234,27 +301,27 @@ int main(int argc, char **argv){
 	}
 
 	// HEKA2ITX hack
-	if (TARGET_TYPE==ITX) {
+	if (TARGET.TYPE==ITX) {
 		// hack: HEKA->ITX conversion will be done in SOPEN	
 		hdr->aECG = dest;
 	}
 
 	hdr = sopen(source, "r", hdr);
 #ifdef WITH_PDP 
-	if (B4C_ERRNUM) {
-		B4C_ERRNUM = 0;  
+	if (hdr->AS.B4C_ERRNUM) {
+		biosigERROR(hdr, 0, NULL);  // reset error 
 		sopen_pdp_read(hdr);
 	}
 #endif
 	// HEKA2ITX hack
-        if (TARGET_TYPE==ITX) {
+        if (TARGET.TYPE==ITX) {
 	if (hdr->TYPE==HEKA) {
 		// hack: HEKA->ITX conversion is already done in SOPEN	
 		dest = NULL; 
 	} 
 	else {
                 fprintf(stdout,"error: only HEKA->ITX is supported - source file is not HEKA file");
-		B4C_ERRNUM = B4C_UNSPECIFIC_ERROR;
+		biosigERROR(hdr, B4C_UNSPECIFIC_ERROR, "error: only HEKA->ITX is supported - source file is not HEKA file");
 	} 
 	}
 
@@ -266,9 +333,9 @@ int main(int argc, char **argv){
 	} 
 #endif
 
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"[112] SOPEN-R finished\n");
+	if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): SOPEN-R finished (error %i)\n",__FILE__,__LINE__, hdr->AS.B4C_ERRNUM);
 
-	if ((status=serror())) {
+	if ((status=serror2(hdr))) {
 		destructHDR(hdr);
 		exit(status); 
 	} 
@@ -280,23 +347,30 @@ int main(int argc, char **argv){
 
 	if ( ( t1 - floor (t1) ) || ( t2 - floor(t2) ) ) {
 		fprintf(stderr,"ERROR SAVE2GDF: cutting from parts of blocks not supported; t1 (%f) and t2 (%f) must be a multiple of block duration %f\n", t1,t2,hdr->SPR / hdr->SampleRate);
-		B4C_ERRNUM = B4C_UNSPECIFIC_ERROR;
-		B4C_ERRMSG = "blocks must not be split";
+		biosigERROR(hdr, B4C_UNSPECIFIC_ERROR, "blocks must not be split");
 	} 
 
-	if ((status=serror())) {
+	if ((status=serror2(hdr))) {
 		destructHDR(hdr);
 		exit(status); 
 	} 
 	
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"[113] SOPEN-R finished\n");
+	if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) SOPEN-R finished\n",__FILE__,__LINE__);
 
-	
-	hdr2ascii(hdr,stdout, FLAG_JSON ? -1 : VERBOSE);
+	sort_eventtable(hdr);
+
+	if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) event table sorted\n",__FILE__,__LINE__);
+
+	if (FLAG_JSON) {
+		fprintf_hdr2json(stdout, hdr);
+	}
+	else {
+		hdr2ascii(hdr, stdout, VERBOSE);
+	}
 
 	// all channels are converted - channel selection currently not supported
     	for (k=0; k<hdr->NS; k++) {
-    		if (!hdr->CHANNEL[k].OnOff && hdr->CHANNEL[k].SPR) {
+		if ( (hdr->CHANNEL[k].OnOff > 0) && hdr->CHANNEL[k].SPR ) {
 			if ((hdr->SPR/hdr->CHANNEL[k].SPR)*hdr->CHANNEL[k].SPR != hdr->SPR)
 				 fprintf(stdout,"Warning: channel %i might be decimated!\n",k+1);
     		};
@@ -309,39 +383,45 @@ int main(int argc, char **argv){
         int flagREREF = 0;
 #endif
 	hdr->FLAG.OVERFLOWDETECTION = 0;
-	hdr->FLAG.UCAL = hdr->FLAG.UCAL && !flagREREF && (TARGET_TYPE==SCP_ECG);
+	hdr->FLAG.UCAL = hdr->FLAG.UCAL && !flagREREF;
 	hdr->FLAG.ROW_BASED_CHANNELS = flagREREF;
 	
 #ifdef CHOLMOD_H
 	if (VERBOSE_LEVEL>7) 
-        	fprintf(stdout,"[121] %p %p Flag.ReRef=%i\n",hdr->Calib, hdr->rerefCHANNEL,flagREREF);
+		fprintf(stdout,"%s (line %i): %p %p Flag.ReRef=%i\n",__FILE__,__LINE__,hdr->Calib, hdr->rerefCHANNEL,flagREREF);
 #endif
 
 	if (VERBOSE_LEVEL>7) 
-		fprintf(stdout,"\n[123] SREAD [%f,%f].\n",t1,t2);
-
-
-	if (t2+t1 > hdr->NRec) t2 = hdr->NRec - t1;
-	if (dest != NULL)
-		count = sread(NULL, t1, t2, hdr);
+		fprintf(stdout,"%s (line %i): SREAD [%f,%f].\n",__FILE__,__LINE__,t1,t2);
 
+	if (hdr->NRec <= 0) { 
+		// in case number of samples is not known
+		count = sread(NULL, t1, (size_t)-1, hdr);
+		t2 = count;
+	}	
+ 	else {
+		if (t2+t1 > hdr->NRec) t2 = hdr->NRec - t1;
+		if ((dest != NULL) || FLAG_CSV || FLAG_DYGRAPH )
+			count = sread(NULL, t1, t2, hdr);
+	}
+	
 	biosig_data_type* data = hdr->data.block;
 	if ((VERBOSE_LEVEL>8) && (hdr->data.size[0]*hdr->data.size[1]>500))
-		fprintf(stdout,"[125] UCAL=%i %e %e %e \n",hdr->FLAG.UCAL,data[100],data[110],data[500+hdr->SPR]);
+		fprintf(stdout,"%s (line %i): UCAL=%i %e %e %e \n",__FILE__,__LINE__,hdr->FLAG.UCAL,data[100],data[110],data[500+hdr->SPR]);
 	
-	if ((status=serror())) {
+	if ((status=serror2(hdr))) {
 		destructHDR(hdr);
 		exit(status);
 	};
 
 	if (VERBOSE_LEVEL>7) 
-		fprintf(stdout,"\n[129] SREAD on %s successful [%i,%i].\n",hdr->FileName,(int)hdr->data.size[0],(int)hdr->data.size[1]);
+		fprintf(stdout,"\n%s (line %i): SREAD on %s successful [%i,%i].\n",__FILE__,__LINE__,hdr->FileName,(int)hdr->data.size[0],(int)hdr->data.size[1]);
 
 //	fprintf(stdout,"\n %f,%f.\n",hdr->FileName,hdr->data.block[3*hdr->SPR],hdr->data.block[4*hdr->SPR]);
 	if (VERBOSE_LEVEL>7) 
-		fprintf(stdout,"\n[130] File  %s =%i/%i\n",hdr->FileName,hdr->FILE.OPEN,hdr->FILE.Des);
+		fprintf(stdout,"\n%s (line %i): File  %s =%i/%i\n",__FILE__,__LINE__,hdr->FileName,hdr->FILE.OPEN,hdr->FILE.Des);
 
-	if (dest==NULL) {
+	if ((dest==NULL) && !FLAG_CSV && !FLAG_DYGRAPH) {
 		if (ne)	/* used for testig SFLUSH_GDF_EVENT_TABLE */
 		{
 			if (hdr->EVENT.N > ne)
@@ -352,25 +432,28 @@ int main(int argc, char **argv){
 			// fprintf(stdout,"Status-SFLUSH %i\n",sflush_gdf_event_table(hdr));
 		}
 
-		if (VERBOSE_LEVEL>7) fprintf(stdout,"[131] going for SCLOSE\n");
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): going for SCLOSE\n",__FILE__,__LINE__);
 		sclose(hdr);
-		if (VERBOSE_LEVEL>7) fprintf(stdout,"[137] SCLOSE(HDR) finished\n");
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): SCLOSE(HDR) finished\n",__FILE__,__LINE__);
+		status=serror2(hdr);
 		destructHDR(hdr);
-		exit(serror());
+		exit(status); 
 	}
 
 	if (hdr->FILE.OPEN) {
 		sclose(hdr); 
 		free(hdr->AS.Header);
 		hdr->AS.Header = NULL;
-		if (VERBOSE_LEVEL>7) fprintf(stdout,"[138] file closed\n");
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): file closed\n",__FILE__,__LINE__);
 	}
 	if (VERBOSE_LEVEL>7 ) 
-		fprintf(stdout,"\n[139] File %s closed sd=%i/%i\n",hdr->FileName,hdr->FILE.OPEN,hdr->FILE.Des);
+		fprintf(stdout,"\n%s (line %i): File %s closed sd=%i/%i\n",__FILE__,__LINE__,hdr->FileName,hdr->FILE.OPEN,hdr->FILE.Des);
 
 	SOURCE_TYPE = hdr->TYPE;
-	hdr->TYPE = TARGET_TYPE;
-	if ((hdr->TYPE==GDF) && (hdr->VERSION<2)) hdr->VERSION = 2.0;
+	if (FLAG_DYGRAPH) TARGET.TYPE=SOURCE_TYPE;
+
+	hdr->TYPE = TARGET.TYPE;
+	if (TARGET.VERSION>=0) hdr->VERSION = TARGET.VERSION;
 
 	hdr->FILE.COMPRESSION = COMPRESSION_LEVEL;
 
@@ -383,7 +466,7 @@ int main(int argc, char **argv){
     		for (k=0; k<hdr->NS; k++)
 		    	if (hdr->CHANNEL[k].OnOff && hdr->CHANNEL[k].SPR) 
 				asGCD = gcd(asGCD, hdr->CHANNEL[k].SPR);
-		if (TARGET_TYPE==EDF) {
+		if (TARGET.TYPE==EDF) {
 			double d = asGCD / hdr->SampleRate;
 			if (d==ceil(d)) asGCD = d; 	// make block duration 1 second	
 		}
@@ -403,7 +486,7 @@ int main(int argc, char **argv){
     *********************************/
 
 #ifdef CHOLMOD_H
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"[199] %p %p\n",hdr->CHANNEL,hdr->rerefCHANNEL);
+	if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): %p %p\n",__FILE__,__LINE__,hdr->CHANNEL,hdr->rerefCHANNEL);
 
         if (hdr->Calib && hdr->rerefCHANNEL) {
 	        if (VERBOSE_LEVEL>6) 
@@ -448,9 +531,9 @@ int main(int argc, char **argv){
 #endif
 	typeof(hdr->NS) k2=0;
     	for (k=0; k<hdr->NS; k++)
-    	if (hdr->CHANNEL[k].OnOff && hdr->CHANNEL[k].SPR) 
-    	{
-	if (VERBOSE_LEVEL > 7) fprintf(stdout,"[204] #%i\n",k);
+    	if (hdr->CHANNEL[k].OnOff && hdr->CHANNEL[k].SPR) {
+
+		if (VERBOSE_LEVEL > 7) fprintf(stdout,"%s (line %i): #%i %i %i N=%i [%i,%i]\n",__FILE__,__LINE__,(int)k,(int)k2,(int)hdr->FLAG.ROW_BASED_CHANNELS,(int)N,(int)hdr->data.size[0],(int)(hdr->data.size[1]));
 	
 		double MaxValue;
 		double MinValue;
@@ -478,8 +561,6 @@ int main(int argc, char **argv){
 		}
 
 		if (!hdr->FLAG.UCAL) {
-			if (PhysMaxValue0 < val) PhysMaxValue0 = val;
-			if (PhysMinValue0 > val) PhysMinValue0 = val;
 			MaxValueF = MaxValue;
 			MinValueF = MinValue;
 			MaxValueD = (MaxValue - hdr->CHANNEL[k].Off) / hdr->CHANNEL[k].Cal;
@@ -487,28 +568,33 @@ int main(int argc, char **argv){
 		} 
 		else {
 			MaxValueF = MaxValue * hdr->CHANNEL[k].Cal + hdr->CHANNEL[k].Off;
-			if (PhysMaxValue0 < MaxValueF) PhysMaxValue0 = MaxValueF;
 			MinValueF = MinValue * hdr->CHANNEL[k].Cal + hdr->CHANNEL[k].Off;
-			if (PhysMinValue0 > MinValue) PhysMinValue0 = MinValueF;
 			MaxValueD = MaxValue;
 			MinValueD = MinValue;
 		}
+		if (PhysMaxValue0 < MaxValueF) PhysMaxValue0 = MaxValueF;
+		if (PhysMinValue0 > MinValueF) PhysMinValue0 = MinValueF;
 
-		if ((SOURCE_TYPE==alpha) && (hdr->CHANNEL[k].GDFTYP==(255+12)) && (TARGET_TYPE==GDF)) 
+		if ((SOURCE_TYPE==alpha) && (hdr->CHANNEL[k].GDFTYP==(255+12)) && (TARGET.TYPE==GDF))
 			// 12 bit into 16 bit 
 			; //hdr->CHANNEL[k].GDFTYP = 3;
-		else if ((SOURCE_TYPE==ETG4000) && (TARGET_TYPE==GDF)) {
+		else if ((SOURCE_TYPE==ETG4000) && (TARGET.TYPE==GDF)) {
 			hdr->CHANNEL[k].GDFTYP  = 16;
 			hdr->CHANNEL[k].PhysMax = MaxValueF;
 			hdr->CHANNEL[k].PhysMin = MinValueF;
 			hdr->CHANNEL[k].DigMax  = MaxValueD;
 			hdr->CHANNEL[k].DigMin  = MinValueD;
 		}
-		else if ((SOURCE_TYPE==GDF) && (TARGET_TYPE==GDF)) 
+/* TODO: check whether this is really needed - was probably just a workaround for another bug
+		
+		else if ((SOURCE_TYPE==GDF) && (TARGET.TYPE==GDF))
 			;
 		else if (SOURCE_TYPE==HEKA)
 			;
-		else if (TARGET_TYPE==SCP_ECG && !hdr->FLAG.UCAL) {
+		else if (SOURCE_TYPE==ABF)
+			;
+*/
+		else if (TARGET.TYPE==SCP_ECG && !hdr->FLAG.UCAL) {
 			double scale = PhysDimScale(hdr->CHANNEL[k].PhysDimCode) *1e9;
 			if (hdr->FLAG.ROW_BASED_CHANNELS) {
 				for (k1=0; k1<N; k1++)
@@ -526,7 +612,7 @@ int main(int argc, char **argv){
 	    		hdr->CHANNEL[k].PhysMax = PhysMax;
 	    		hdr->CHANNEL[k].PhysMin = -PhysMax;
 		}
-		else if (TARGET_TYPE==EDF) {
+		else if (TARGET.TYPE==EDF) {
 	    		hdr->CHANNEL[k].GDFTYP = 3;
 	    		hdr->CHANNEL[k].DigMax = ldexp(1.0,15)-1.0;
 	    		hdr->CHANNEL[k].DigMin = -hdr->CHANNEL[k].DigMax;
@@ -535,19 +621,22 @@ int main(int argc, char **argv){
 	    		hdr->CHANNEL[k].Cal = (hdr->CHANNEL[k].PhysMax - hdr->CHANNEL[k].PhysMin) / (hdr->CHANNEL[k].DigMax - hdr->CHANNEL[k].DigMin);
 	    		hdr->CHANNEL[k].Off = hdr->CHANNEL[k].PhysMin - hdr->CHANNEL[k].DigMin * hdr->CHANNEL[k].Cal;
 		}
-		else if ((hdr->CHANNEL[k].GDFTYP<10 ) && (TARGET_TYPE==GDF || TARGET_TYPE==CFWB)) {
+		else if ((hdr->CHANNEL[k].GDFTYP<10 ) && (TARGET.TYPE==GDF || TARGET.TYPE==CFWB)) {
 			/* heuristic to determine optimal data type */
-			if ((MaxValue <= 127) && (MinValue >= -128))
+
+			if (VERBOSE_LEVEL > 7) fprintf(stdout,"%s (line %i): #%i %i %f %f %f %f %f %f\n",__FILE__,__LINE__,k+1,(int)hdr->FLAG.UCAL,MinValue,MaxValue,MinValueF,MaxValueF,MinValueD,MaxValueD);
+
+			if ((MaxValueD <= 127) && (MinValueD >= -128))
 		    		hdr->CHANNEL[k].GDFTYP = 1;
-			else if ((MaxValue <= 255.0) && (MinValue >= 0.0))
+			else if ((MaxValueD <= 255.0) && (MinValueD >= 0.0))
 			    	hdr->CHANNEL[k].GDFTYP = 2;
-			else if ((MaxValue <= ldexp(1.0,15)-1.0) && (MinValue >= ldexp(-1.0,15)))
+			else if ((MaxValueD <= ldexp(1.0,15)-1.0) && (MinValueD >= ldexp(-1.0,15)))
 		    		hdr->CHANNEL[k].GDFTYP = 3;
-			else if ((MaxValue <= ldexp(1.0,16)-1.0) && (MinValue >= 0.0))
+			else if ((MaxValueD <= ldexp(1.0,16)-1.0) && (MinValueD >= 0.0))
 			    	hdr->CHANNEL[k].GDFTYP = 4;
-			else if ((MaxValue <= ldexp(1.0,31)-1.0) && (MinValue >= ldexp(-1.0,31)))
+			else if ((MaxValueD <= ldexp(1.0,31)-1.0) && (MinValueD >= ldexp(-1.0,31)))
 		    		hdr->CHANNEL[k].GDFTYP = 5;
-			else if ((MaxValue <= ldexp(1.0,32)-1.0) && (MinValue >= 0.0))
+			else if ((MaxValueD <= ldexp(1.0,32)-1.0) && (MinValueD >= 0.0))
 		    		hdr->CHANNEL[k].GDFTYP = 6;
 		} 
 		else {
@@ -558,48 +647,123 @@ int main(int argc, char **argv){
 		k2++;
 	}
 	if (!FLAG_CONVERSION_TESTED) 
-		fprintf(stderr,"Warning SAVE2GDF: conversion from %s to %s not tested\n",GetFileTypeString(SOURCE_TYPE),GetFileTypeString(TARGET_TYPE));
+		fprintf(stderr,"Warning SAVE2GDF: conversion from %s to %s not tested\n",GetFileTypeString(SOURCE_TYPE),GetFileTypeString(TARGET.TYPE));
 
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"[205] UCAL=%i\n", hdr->FLAG.UCAL);
+	if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): UCAL=%i\n",__FILE__,__LINE__, hdr->FLAG.UCAL);
 
-	/* write file */
-	size_t destlen = strlen(dest); 
-	char *tmp = (char*)malloc(destlen+4);
-	strcpy(tmp,dest);
-	if (hdr->FILE.COMPRESSION)  // add .gz extension to filename  
+	hdr->FLAG.ANONYMOUS = FLAG_ANON;
+
+	/*
+	  keep header data from previous file, in might contain optional data
+	  (GDF Header3, EventDescription, hdr->SCP.SectionX, etc. )
+	  and might still be referenced and needed.
+	*/
+	void *tmpmem = hdr->AS.Header;
+	hdr->AS.Header = NULL;
+
+	if (FLAG_CSV) {
+		const char SEP=',';
+		FILE *fid = stdout;
+		if (dest != NULL) fid = fopen(dest,"wt+");
+		ssize_t k1;
+		size_t k2;
+		char flag = 0;
+		for (k2 = 0; k2 < hdr->NS; k2++) {
+			CHANNEL_TYPE *hc = hdr->CHANNEL+k2;
+			if (hc->OnOff) {
+				if (flag) fprintf(fid,"%c",SEP);
+				flag = 1;
+				fprintf(fid,"\"%s [%s]\"", hc->Label, PhysDim3(hc->PhysDimCode) );
+			}
+		}
+		for (k1 = 0; k1 < hdr->SPR*hdr->NRec; k1++) {
+			flag = 0;
+			for (k2 = 0; k2 < hdr->NS; k2++) {
+				CHANNEL_TYPE *hc = hdr->CHANNEL+k2;
+				if (hc->OnOff) {
+					fprintf(fid,"%c", flag ? SEP : '\n');
+					flag = 1;
+					size_t p = hdr->FLAG.ROW_BASED_CHANNELS ? hdr->data.size[0] * k1 + k2 : hdr->data.size[0] * k2 + k1 ;
+					fprintf(fid,"%g", data[p]);
+				}
+			}
+		}
+		fprintf(fid,"\n");
+		if (dest != NULL) fclose(fid);
+	}
+	else if (FLAG_DYGRAPH) {
+		FILE *fid = stdout;
+		if (dest != NULL) fid = fopen(dest,"wb+");
+		fprintf(fid,"{\n\"Header\": ");
+		fprintf_hdr2json(fid, hdr);
+		fprintf(fid,",\n\"Data\": [ ");
+		ssize_t k1;
+		size_t k2;
+		for (k1=0; k1 < hdr->SPR * hdr->NRec; k1++) {
+			if (k1>0) fprintf(fid,",\n\t");
+			fprintf(fid,"[");
+			for (k2=0; k2<hdr->NS; k2++) {
+				if (k2>0) fprintf(fid,", ");
+				size_t p = hdr->FLAG.ROW_BASED_CHANNELS ? hdr->data.size[0] * k1 + k2 : hdr->data.size[0] * k2 + k1 ;
+				fprintf(fid,"%g",data[p]);
+			}
+			fprintf(fid,"]");
+		}
+
+		fprintf(fid," ],\n\"labels\": [ ");
+		for (k2=0; k2<hdr->NS; k2++) {
+			CHANNEL_TYPE *hc = hdr->CHANNEL+k2;
+			if (k2>0) fprintf(fid,", ");
+			fprintf(fid,"\"%s [%s]\"",hc->Label,PhysDim3(hc->PhysDimCode));
+		}
+		fprintf(fid,"]\n}\n");
+		if (dest != NULL) fclose(fid);
+	}
+	else {
+		/* write file */
+		size_t destlen = strlen(dest); 
+		char *tmp = (char*)malloc(destlen+4);
+		strcpy(tmp,dest);
+		if (hdr->FILE.COMPRESSION)  // add .gz extension to filename  
 		strcpy(tmp+destlen,".gz");
 
 	if (VERBOSE_LEVEL>7) 
-		fprintf(stdout,"[211] z=%i sd=%i\n",hdr->FILE.COMPRESSION,hdr->FILE.Des);
+		fprintf(stdout,"%s (line %i) z=%i sd=%i\n",__FILE__,__LINE__,hdr->FILE.COMPRESSION,hdr->FILE.Des);
 
-	hdr->FLAG.ANONYMOUS = 1; 	// no personal names are processed 
+		sopen(tmp, "wb", hdr);
 
-	hdr = sopen(tmp, "wb", hdr);
-	free(tmp); 
-	if ((status=serror())) {
-		destructHDR(hdr);
-		exit(status); 
-	}	
+	if (VERBOSE_LEVEL>7) fprintf(stdout,"returned from sopen-wb\n");
+
+		free(tmp); 
+		if ((status=serror2(hdr))) {
+			destructHDR(hdr);
+			exit(status); 
+		}	
 #ifndef WITHOUT_NETWORK
-	if (hdr->FILE.Des>0) 
-		savelink(source);
-#endif 
-	if (VERBOSE_LEVEL>7)
-		fprintf(stdout,"\n[221] File %s opened. %i %i %i Des=%i\n",hdr->FileName,hdr->AS.bpb,hdr->NS,(int)(hdr->NRec),hdr->FILE.Des);
+		if (hdr->FILE.Des>0) 
+			savelink(source);
+#endif
+		if (VERBOSE_LEVEL>7)
+			fprintf(stdout,"\n%s (line %i): File %s opened. %i %i %i Des=%i\n",__FILE__,__LINE__,hdr->FileName,hdr->AS.bpb,hdr->NS,(int)(hdr->NRec),hdr->FILE.Des);
 
-	swrite(data, hdr->NRec, hdr);
+		swrite(data, hdr->NRec, hdr);
 
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"[231] SWRITE finishes\n");
-	if ((status=serror())) { 
-		destructHDR(hdr);
-		exit(status); 
-    	}	
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): SWRITE finishes (errno=%i)\n",__FILE__,__LINE__,(int)hdr->AS.B4C_ERRNUM);
+		if ((status=serror2(hdr))) { 
+			destructHDR(hdr);
+			exit(status); 
+	    	}	
 
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"[236] SCLOSE finished\n");
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): SWRITE finishes %i\n",__FILE__,__LINE__,(int)status);
 
-	sclose(hdr);
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"[241] SCLOSE finished\n");
+		sclose(hdr);
+
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): SCLOSE finished\n",__FILE__,__LINE__);
+
+	}
+	status = serror2(hdr);
 	destructHDR(hdr);
-	exit(serror()); 
+	if (tmpmem != NULL) free(tmpmem);
+	exit(status); 
 }
 
